#!/usr/bin/env python

import math
# Hamburger
hamb = {'fat': 9,
        'carb': 31,
        'prot': 12,
        'calc': 0.1,
        'weight': 100,
        'price': 34}      

# Box of 6 Chicken McNuggets
nugget = {'fat': 18,
          'carb': 18,
          'prot': 13,
          'calc': 0.02,
          'weight': 97,
          'price': 89}

def line_axis_intersection(inequalitys):
    '''Return points of intersection of lines with x/y axis.'''
    points = []
    for i in inequalitys:
        points.extend([(i[2]/i[0], 0), (0, i[2]/i[1])])
    return points

def lines_intersection(inequalitys):
    '''Return points of intersection between lines.'''
    points = []
    for i, line1 in enumerate(inequalitys):
        for line2 in inequalitys[i+1:]:
            result = lines_intersection_point(*(line1[:-1] + line2[:-1]))
            if result is not None:
                points.append(result)
    return points

def lines_intersection_point(a0, b0, c0, a1, b1, c1):
    '''Return point of intersection between two lines. Return None if one of the
       lines does not exist or lines are parallel.

    '''
    if a0 == b0 == 0 or a1 == b1 == 0 or a0*b1 - b0*a1 == 0:
        return None

    x = (b1*c0 - b0*c1)/(a0*b1 - b0*a1)
    y = (a0*c1 - a1*c0)/(a0*b1 - b0*a1)
    return x, y

def feasible_points(inequalitys):
    '''Return set of feasible points'''
    points = line_axis_intersection(inequalitys)
    points.extend(lines_intersection(inequalitys))
    points[:] = [p for p in points if p[0] >= 0 and p[1] >= 0]
    points[:] = [p for p in points if point_filter(p, inequalitys)]
    return points

def point_filter(point, inequalitys):
    '''Is the point within the set of feasible solutions? Return True if yes.'''
    for i in inequalitys:
        if comparison(*(point + i)) == False:
            return False
    return True

def comparison(hamb, nugget, a, b, c, sign):
    '''Return True if point satisfy the inequality'''
    if sign == 'more':
        return round(hamb*a + nugget*b, 2) >= c
    elif sign == 'less':
        return round(hamb*a + nugget*b, 2) <= c
    else: return None

def cat_food_table(food_needs, overeating, nutr_needs):
    '''Generate inequalitys for the set of feasible solutions.'''
    global hamb, nugget

    inequalitys = ((hamb['fat'], nugget['fat'], nutr_needs['fat'], 'more'),
                   (hamb['carb'], nugget['carb'], nutr_needs['carb'], 'more'),
                   (hamb['prot'], nugget['prot'], nutr_needs['prot'], 'more'),
                   (hamb['calc'], nugget['calc'], nutr_needs['calc'], 'more'),
                   (hamb['weight'], nugget['weight'], food_needs, 'more'),
                   (hamb['weight'], nugget['weight'], overeating, 'less'))

    print('-'*37 + '\n')
    print('Daily food requirements:\n')
    print('minimum amount of food: {0:d} g'.format(food_needs))
    print('maximum amount of food: {0:d} g'.format(overeating))
    print('-'*37 + '\n')
    print('Daily nutritional requirements:\n')
    print('Fat:\t\t{0} g'.format(nutr_needs['fat']))
    print('Carbs:\t\t{0} g'.format(nutr_needs['carb']))
    print('Protein:\t{0} g'.format(nutr_needs['prot']))
    print('Calcium:\t{0} g'.format(nutr_needs['calc']))
    print('*'*37 + '\n')

    return inequalitys

def food_calculator(hambs, nuggets):
    '''Return number of hamburgers and chicken McNuggets eaten by cat. This
       function is really ugly. Sorry.

    '''
    global hamb, nugget
    rem_hamb, whole_hamb = math.modf(hambs)  # remainder and whole hamburger
    rem_nugg, whole_nugg = math.modf(nuggets)  # remainder and whole six_nugget
    rem_hamb = round(rem_hamb, 2)
    rem_nugg = round(rem_nugg, 2)

    if rem_hamb != 0 and rem_nugg != 0:
        add_hamb = True  # if true add hamburger instead remainders
        add_nugget = True  # if true add six_nuggets instead remainders
        blend = {'fat': rem_hamb*hamb['fat'] + rem_nugg*nugget['fat'],
                 'carb': rem_hamb*hamb['carb'] + rem_nugg*nugget['carb'],
                 'prot': rem_hamb*hamb['prot'] + rem_nugg*nugget['prot'],
                 'calc': rem_hamb*hamb['calc'] + rem_nugg*nugget['calc'],
                }
        for b, h in zip(blend.values(), hamb.values()):
            if b <= h and add_hamb:
                add_hamb = True
            else:
                add_hamb = False
                break
        if add_hamb:
            whole_hamb += 1
            print_eaten_food(whole_hamb, whole_nugg)
            return whole_hamb, whole_nugg  # hamburger +1
        else:
            for b, n in zip(blend.values(), nugget.values()):
                if b <= n and add_nugget:
                    add_nugget = True
                else:
                    add_nugget = False
                    break
            if add_nugget:
                whole_nugg += 1
                print_eaten_food(whole_hamb, whole_nugg)
                return whole_hamb, whole_nugg  # box of 6 nuggets +1
            else:
                whole_hamb += 1
                whole_nugg += 1
                print_eaten_food(whole_hamb, whole_nugg)
                return whole_hamb, whole_nugg  # box +1 and hamburger +1
    elif rem_hamb == 0 and rem_nugg != 0:
      whole_nugg += 1
      print_eaten_food(whole_hamb, whole_nugg)
      return whole_hamb, whole_nugg  # box of 6 nuggets +1
    elif rem_hamb != 0 and rem_nugg == 0:
      whole_hamb += 1
      print_eaten_food(whole_hamb, whole_nugg)
      return whole_hamb, whole_nugg  # hamburger +1
    else:
        print_eaten_food(whole_hamb, whole_nugg)
        return whole_hamb, whole_nugg  # nothing to append

def print_eaten_food(hambs, nuggets):
    '''Just print number of hamburgers and chicken McNuggets and their cost'''
    global hamb, nugget
    total_cost = hamb['price']*hambs + nugget['price']*nuggets
    print('Cat`s diet:\n\nHamburgers:\t\t{0}\nChicken McNuggets:\t{1}'.format(
          int(hambs), int(nuggets)))
    print('Total cost:\t\t{0} RUB'.format(int(total_cost)))
    print('*'*37 + '\n')



def consumed_nutritions(hamb, nugget, inequalitys, nutr_needs):
    '''Show table of consumed nutriations.'''
    consumed_nutr = {'fat': hamb*inequalitys[0][0] + nugget*inequalitys[0][1],
                     'carb': hamb*inequalitys[1][0] + nugget*inequalitys[1][1],
                     'prot': hamb*inequalitys[2][0] + nugget*inequalitys[2][1],
                     'calc': hamb*inequalitys[3][0] + nugget*inequalitys[3][1]}
    print('Consumed nutrients:\n')
    print('Fat:\t\t{0} g\t(+{1} g)'.format(round(consumed_nutr['fat'], 1),
          round(abs(consumed_nutr['fat'] - nutr_needs['fat']), 1)))
    print('Carbs:\t\t{0} g\t(+{1} g)'.format(round(consumed_nutr['carb'], 1),
          round(abs(consumed_nutr['carb'] - nutr_needs['carb']), 1)))
    print('Protein:\t{0} g\t(+{1} g)'.format(round(consumed_nutr['prot'], 1),
          round(abs(consumed_nutr['prot'] - nutr_needs['prot']), 1)))
    print('Calcium:\t{0} g\t(+{1} g)'.format(round(consumed_nutr['calc'], 1),
          round(abs(consumed_nutr['calc'] - nutr_needs['calc']), 1)))
    print('*'*37)

def main():
    global hamb, nugget
    cat_weight = 3000 # Cats weight in gram
    print('\nCat`s weight:\t{0} kg'.format(round(cat_weight/1000, 2)))
    food_needs = round(5*cat_weight/100)  # Necessary min of food
    overeating = round(10*cat_weight/100)  # Max of food

    nutr_needs = {'fat': round(30*food_needs/100, 1),
                  'carb': round(16.7*food_needs/100, 1),
                  'prot': round(46.7*food_needs/100, 1),
                  'calc': round(2*food_needs/100, 1)}

    inequalitys = cat_food_table(food_needs, overeating, nutr_needs)
    points = feasible_points(inequalitys)

    if points != []:
        diet_price = lambda h, n: hamb['price']*h + nugget['price']*n
        prices = [diet_price(*p) for p in points]
        solution_point = points[prices.index(min(prices))]
        order = food_calculator(solution_point[0], solution_point[1])
        consumed_nutritions(order[0], order[1], inequalitys, nutr_needs)
    else: print('No solution')

if __name__ == '__main__':
    main()